from __future__ import unicode_literals

TEST_EMAIL_ADDRESS = 'test@example.com'
TEST_RECIPIENTS_MULTIPLE_COMMA = 'test@example.com,test2@example.com'
TEST_RECIPIENTS_MULTIPLE_SEMICOLON = 'test@example.com;test2@example.com'
TEST_RECIPIENTS_MULTIPLE_MIXED = 'test@example.com,test2@example.com;test2@example.com'
TEST_RECIPIENTS_MULTIPLE_MIXED_LIST = (
    'test@example.com', 'test2@example.com', 'test2@example.com',
)
TEST_USER_MAILER_LABEL = 'test user mailer label'
TEST_USER_MAILER_BACKEND_PATH = 'mailer.tests.mailers.TestBackend'
